/*
 * uart_demo.c
 *
 *  Created on: Feb 12, 2023
 *      Author: Harry L.
 */


#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "main.h"

#include "uart_lib.h"


/******************************************************************
 *
 * define the UART-Object
 * max. number of UARTs is defined in uart_lib.h
 * default is 3 UARTs
 *
 ******************************************************************/
UART_com_t myUART;

/******************************************************************
 *
 * define size of RX- and TX-Buffer and allocate them
 *
 ******************************************************************/
#define TX_BUFFER_SIZE 100
#define RX_BUFFER_SIZE 100

uint8_t TxBuffer[TX_BUFFER_SIZE];
uint8_t RxBuffer[RX_BUFFER_SIZE];

/******************************************************************
 *
 * This function is called, whenever a complete line was received
 *
 ******************************************************************/
void LineReceivedCallback(uint8_t *line)
{
	uart_puts(&myUART, "Received line: ");
	uart_puts(&myUART, (char *)line);
	uart_puts(&myUART, "\r\n\n>");
}

/******************************************************************
 *
 * Init and start the UART-system
 *
 ******************************************************************/
void myUART_LineDemo(UART_HandleTypeDef* huart)
{
	myUART.huart = huart;						// the handle of the used u(s)art

	myUART.TX_FiFo = TxBuffer;					// the buffers for RX and TX
	myUART.TX_FiFo_Size = TX_BUFFER_SIZE;
	myUART.RX_Buffer = RxBuffer;
	myUART.RX_Buffer_Size = RX_BUFFER_SIZE;

	myUART.data_type = string;					// mode maybe raw, data or string
												// in string-mode the callback is fired, when the term_char has been received.
												// in string mode, special chars like BS are also handled
												// the term_char is not part of the final string

												// in raw-mode the callback is fired after any received byte.

												// the data-mode, callback is executed when term_char is received
												// term_char is the last byte of the array
												// and the size is also passed to the callback

	myUART.Echo = true;							// should the input be echoed on terminal (only used in string-mode)
	myUART.term_char = '\r';

	myUART.StringCpltCallback = &LineReceivedCallback;	// define the function which is called, when the line has been completed

	uart_init(&myUART);						// initialize our u(s)art

	uart_puts(&myUART, "\r\n\nInit started\r\nTyoe in a line!\r\nYou may use BS to edit faulty inputs\r\nLet's go!\r\n\n>");
}

