/*
 * uart_lib.h
 *
 *  Created on: 11.02.2023
 *      Author: Harry L.
 *
 *  Ver. 1.01
 *
 */

#ifndef UART_LIB_H_
#define UART_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif

/****************************************************************
 *
 * Maximum number of usable UARTs
 *
 ****************************************************************/
#define MAX_COM_PORTS	3

/****************************************************************
 *
 * Don't change anything below!
 *
 ****************************************************************/

typedef struct
{
	uint8_t rxbuf;
	size_t buffer_p;
	volatile bool uart_idle;
	volatile bool line_complete;
	size_t fifo_get;
	size_t fifo_put;
} Serial_COM_Instance_t;

typedef enum
{
	raw,		// single byte
	data,		// array of bytes
	string		// zero-terminated string
} data_type_t;

/********************************************************************
 *
 * This struct has to be created by user, and has to be populated
 * before calling any API-function!
 * see uart_demo.c for example usage
 *
 ********************************************************************/

typedef struct
{
	UART_HandleTypeDef *huart;
	uint8_t *RX_Buffer;				// Pointer to rx-buffer
	size_t RX_Buffer_Size;			// size of rx-buffer
	uint8_t *TX_FiFo;				// pointer to tx-buffer
	size_t TX_FiFo_Size;			// size of tx-buffer
	data_type_t data_type;			// type of input-data
	uint8_t term_char;				// EOL-Char in line-mode
	bool Echo;						// only valid in string-mode
									// false -> line-input not editable, no echo
									// true  -> line-input editable with echo
	union							// Callback-function
	{
		void (*StringCpltCallback)(uint8_t *data);
		void (*DataCpltCallback)(uint8_t *data, size_t cnt);
		void (*ByteCpltCallback)(uint8_t data);
		void *callback;
	};
	Serial_COM_Instance_t Instance;		// internal data
} UART_com_t;

/********************************************************************
 *
 * UART API-Fuctions
 *
 ********************************************************************/

void uart_init(UART_com_t *Serial_COM);

// send binary data
void uart_put_byte(UART_com_t *Serial_COM, uint8_t c);
void uart_put_data(UART_com_t *Serial_COM, uint8_t *s, size_t cnt);

// send char and strings
void uart_putc(UART_com_t *Serial_COM, char c);
void uart_puts(UART_com_t *Serial_COM, char *s);

#ifdef __cplusplus
}
#endif

#endif /* UART_LIB_H_ */
